# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _gridfunc
else:
    import _gridfunc

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _gridfunc.SWIG_PyInstanceMethod_New
_swig_new_static_method = _gridfunc.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

class intp(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _gridfunc.intp_swiginit(self, _gridfunc.new_intp())
    __swig_destroy__ = _gridfunc.delete_intp

    def assign(self, value):
        return _gridfunc.intp_assign(self, value)
    assign = _swig_new_instance_method(_gridfunc.intp_assign)

    def value(self):
        return _gridfunc.intp_value(self)
    value = _swig_new_instance_method(_gridfunc.intp_value)

    def cast(self):
        return _gridfunc.intp_cast(self)
    cast = _swig_new_instance_method(_gridfunc.intp_cast)

    @staticmethod
    def frompointer(t):
        return _gridfunc.intp_frompointer(t)
    frompointer = _swig_new_static_method(_gridfunc.intp_frompointer)

# Register intp in _gridfunc:
_gridfunc.intp_swigregister(intp)

def intp_frompointer(t):
    return _gridfunc.intp_frompointer(t)
intp_frompointer = _gridfunc.intp_frompointer

class doublep(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        _gridfunc.doublep_swiginit(self, _gridfunc.new_doublep())
    __swig_destroy__ = _gridfunc.delete_doublep

    def assign(self, value):
        return _gridfunc.doublep_assign(self, value)
    assign = _swig_new_instance_method(_gridfunc.doublep_assign)

    def value(self):
        return _gridfunc.doublep_value(self)
    value = _swig_new_instance_method(_gridfunc.doublep_value)

    def cast(self):
        return _gridfunc.doublep_cast(self)
    cast = _swig_new_instance_method(_gridfunc.doublep_cast)

    @staticmethod
    def frompointer(t):
        return _gridfunc.doublep_frompointer(t)
    frompointer = _swig_new_static_method(_gridfunc.doublep_frompointer)

# Register doublep in _gridfunc:
_gridfunc.doublep_swigregister(doublep)

def doublep_frompointer(t):
    return _gridfunc.doublep_frompointer(t)
doublep_frompointer = _gridfunc.doublep_frompointer

import mfem._par.array
import mfem._par.mem_manager
import mfem._par.vector
import mfem._par.coefficient
import mfem._par.globals
import mfem._par.matrix
import mfem._par.operators
import mfem._par.intrules
import mfem._par.sparsemat
import mfem._par.densemat
import mfem._par.eltrans
import mfem._par.fe
import mfem._par.geom
import mfem._par.fespace
import mfem._par.mesh
import mfem._par.sort_pairs
import mfem._par.ncmesh
import mfem._par.vtk
import mfem._par.element
import mfem._par.table
import mfem._par.hash
import mfem._par.vertex
import mfem._par.fe_coll
import mfem._par.lininteg
import mfem._par.handle
import mfem._par.hypre
import mfem._par.restriction
import mfem._par.bilininteg
import mfem._par.linearform
import mfem._par.nonlininteg
class GridFunction(mfem._par.vector.Vector):
    r"""Proxy of C++ mfem::GridFunction class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def MakeOwner(self, _fec):
        r"""MakeOwner(GridFunction self, FiniteElementCollection _fec)"""
        return _gridfunc.GridFunction_MakeOwner(self, _fec)
    MakeOwner = _swig_new_instance_method(_gridfunc.GridFunction_MakeOwner)

    def OwnFEC(self):
        r"""OwnFEC(GridFunction self) -> FiniteElementCollection"""
        return _gridfunc.GridFunction_OwnFEC(self)
    OwnFEC = _swig_new_instance_method(_gridfunc.GridFunction_OwnFEC)

    def VectorDim(self):
        r"""VectorDim(GridFunction self) -> int"""
        return _gridfunc.GridFunction_VectorDim(self)
    VectorDim = _swig_new_instance_method(_gridfunc.GridFunction_VectorDim)

    def GetTrueVector(self, *args):
        r"""
        GetTrueVector(GridFunction self) -> Vector
        GetTrueVector(GridFunction self) -> Vector
        """
        return _gridfunc.GridFunction_GetTrueVector(self, *args)
    GetTrueVector = _swig_new_instance_method(_gridfunc.GridFunction_GetTrueVector)

    def GetTrueDofs(self, tv):
        r"""GetTrueDofs(GridFunction self, Vector tv)"""
        return _gridfunc.GridFunction_GetTrueDofs(self, tv)
    GetTrueDofs = _swig_new_instance_method(_gridfunc.GridFunction_GetTrueDofs)

    def SetTrueVector(self):
        r"""SetTrueVector(GridFunction self)"""
        return _gridfunc.GridFunction_SetTrueVector(self)
    SetTrueVector = _swig_new_instance_method(_gridfunc.GridFunction_SetTrueVector)

    def SetFromTrueDofs(self, tv):
        r"""SetFromTrueDofs(GridFunction self, Vector tv)"""
        return _gridfunc.GridFunction_SetFromTrueDofs(self, tv)
    SetFromTrueDofs = _swig_new_instance_method(_gridfunc.GridFunction_SetFromTrueDofs)

    def SetFromTrueVector(self):
        r"""SetFromTrueVector(GridFunction self)"""
        return _gridfunc.GridFunction_SetFromTrueVector(self)
    SetFromTrueVector = _swig_new_instance_method(_gridfunc.GridFunction_SetFromTrueVector)

    def GetValue(self, *args):
        r"""
        GetValue(GridFunction self, int i, IntegrationPoint ip, int vdim=1) -> double
        GetValue(GridFunction self, ElementTransformation T, IntegrationPoint ip, int comp=0, Vector tr=None) -> double
        """
        return _gridfunc.GridFunction_GetValue(self, *args)
    GetValue = _swig_new_instance_method(_gridfunc.GridFunction_GetValue)

    def GetVectorValue(self, *args):
        r"""
        GetVectorValue(GridFunction self, int i, IntegrationPoint ip, Vector val)
        GetVectorValue(GridFunction self, ElementTransformation T, IntegrationPoint ip, Vector val, Vector tr=None)
        """
        return _gridfunc.GridFunction_GetVectorValue(self, *args)
    GetVectorValue = _swig_new_instance_method(_gridfunc.GridFunction_GetVectorValue)

    def GetValues(self, *args):
        r"""
        GetValues(GridFunction self, int i, IntegrationRule ir, Vector vals, int vdim=1)
        GetValues(GridFunction self, int i, IntegrationRule ir, Vector vals, DenseMatrix tr, int vdim=1)
        GetValues(GridFunction self, ElementTransformation T, IntegrationRule ir, Vector vals, int comp=0, DenseMatrix tr=None)
        """
        return _gridfunc.GridFunction_GetValues(self, *args)
    GetValues = _swig_new_instance_method(_gridfunc.GridFunction_GetValues)

    def GetVectorValues(self, *args):
        r"""
        GetVectorValues(GridFunction self, int i, IntegrationRule ir, DenseMatrix vals, DenseMatrix tr)
        GetVectorValues(GridFunction self, ElementTransformation T, IntegrationRule ir, DenseMatrix vals, DenseMatrix tr=None)
        """
        return _gridfunc.GridFunction_GetVectorValues(self, *args)
    GetVectorValues = _swig_new_instance_method(_gridfunc.GridFunction_GetVectorValues)

    def GetFaceValues(self, i, side, ir, vals, tr, vdim=1):
        r"""GetFaceValues(GridFunction self, int i, int side, IntegrationRule ir, Vector vals, DenseMatrix tr, int vdim=1) -> int"""
        return _gridfunc.GridFunction_GetFaceValues(self, i, side, ir, vals, tr, vdim)
    GetFaceValues = _swig_new_instance_method(_gridfunc.GridFunction_GetFaceValues)

    def GetFaceVectorValues(self, i, side, ir, vals, tr):
        r"""GetFaceVectorValues(GridFunction self, int i, int side, IntegrationRule ir, DenseMatrix vals, DenseMatrix tr) -> int"""
        return _gridfunc.GridFunction_GetFaceVectorValues(self, i, side, ir, vals, tr)
    GetFaceVectorValues = _swig_new_instance_method(_gridfunc.GridFunction_GetFaceVectorValues)

    def GetLaplacians(self, *args):
        r"""
        GetLaplacians(GridFunction self, int i, IntegrationRule ir, Vector laps, int vdim=1)
        GetLaplacians(GridFunction self, int i, IntegrationRule ir, Vector laps, DenseMatrix tr, int vdim=1)
        """
        return _gridfunc.GridFunction_GetLaplacians(self, *args)
    GetLaplacians = _swig_new_instance_method(_gridfunc.GridFunction_GetLaplacians)

    def GetHessians(self, *args):
        r"""
        GetHessians(GridFunction self, int i, IntegrationRule ir, DenseMatrix hess, int vdim=1)
        GetHessians(GridFunction self, int i, IntegrationRule ir, DenseMatrix hess, DenseMatrix tr, int vdim=1)
        """
        return _gridfunc.GridFunction_GetHessians(self, *args)
    GetHessians = _swig_new_instance_method(_gridfunc.GridFunction_GetHessians)

    def GetValuesFrom(self, orig_func):
        r"""GetValuesFrom(GridFunction self, GridFunction orig_func)"""
        return _gridfunc.GridFunction_GetValuesFrom(self, orig_func)
    GetValuesFrom = _swig_new_instance_method(_gridfunc.GridFunction_GetValuesFrom)

    def GetBdrValuesFrom(self, orig_func):
        r"""GetBdrValuesFrom(GridFunction self, GridFunction orig_func)"""
        return _gridfunc.GridFunction_GetBdrValuesFrom(self, orig_func)
    GetBdrValuesFrom = _swig_new_instance_method(_gridfunc.GridFunction_GetBdrValuesFrom)

    def GetVectorFieldValues(self, i, ir, vals, tr, comp=0):
        r"""GetVectorFieldValues(GridFunction self, int i, IntegrationRule ir, DenseMatrix vals, DenseMatrix tr, int comp=0)"""
        return _gridfunc.GridFunction_GetVectorFieldValues(self, i, ir, vals, tr, comp)
    GetVectorFieldValues = _swig_new_instance_method(_gridfunc.GridFunction_GetVectorFieldValues)

    def ReorderByNodes(self):
        r"""ReorderByNodes(GridFunction self)"""
        return _gridfunc.GridFunction_ReorderByNodes(self)
    ReorderByNodes = _swig_new_instance_method(_gridfunc.GridFunction_ReorderByNodes)

    def GetNodalValues(self, *args):
        '''
        GetNodalValues(i)   ->   GetNodalValues(vector, vdim)
        GetNodalValues(i, array<dobule>, vdim)
        '''
        from .vector import Vector
        if len(args) == 1:
            vec = Vector()
            _gridfunc.GridFunction_GetNodalValues(self, vec, args[0])
            vec.thisown = 0
            return vec.GetDataArray()
        else:
            return _gridfunc.GridFunction_GetNodalValues(self, *args)



    def GetVectorFieldNodalValues(self, val, comp):
        r"""GetVectorFieldNodalValues(GridFunction self, Vector val, int comp)"""
        return _gridfunc.GridFunction_GetVectorFieldNodalValues(self, val, comp)
    GetVectorFieldNodalValues = _swig_new_instance_method(_gridfunc.GridFunction_GetVectorFieldNodalValues)

    def ProjectVectorFieldOn(self, vec_field, comp=0):
        r"""ProjectVectorFieldOn(GridFunction self, GridFunction vec_field, int comp=0)"""
        return _gridfunc.GridFunction_ProjectVectorFieldOn(self, vec_field, comp)
    ProjectVectorFieldOn = _swig_new_instance_method(_gridfunc.GridFunction_ProjectVectorFieldOn)

    def GetDerivative(self, comp, der_comp, der):
        r"""GetDerivative(GridFunction self, int comp, int der_comp, GridFunction der)"""
        return _gridfunc.GridFunction_GetDerivative(self, comp, der_comp, der)
    GetDerivative = _swig_new_instance_method(_gridfunc.GridFunction_GetDerivative)

    def GetDivergence(self, tr):
        r"""GetDivergence(GridFunction self, ElementTransformation tr) -> double"""
        return _gridfunc.GridFunction_GetDivergence(self, tr)
    GetDivergence = _swig_new_instance_method(_gridfunc.GridFunction_GetDivergence)

    def GetCurl(self, tr, curl):
        r"""GetCurl(GridFunction self, ElementTransformation tr, Vector curl)"""
        return _gridfunc.GridFunction_GetCurl(self, tr, curl)
    GetCurl = _swig_new_instance_method(_gridfunc.GridFunction_GetCurl)

    def GetGradient(self, tr, grad):
        r"""GetGradient(GridFunction self, ElementTransformation tr, Vector grad)"""
        return _gridfunc.GridFunction_GetGradient(self, tr, grad)
    GetGradient = _swig_new_instance_method(_gridfunc.GridFunction_GetGradient)

    def GetGradients(self, *args):
        r"""
        GetGradients(GridFunction self, ElementTransformation tr, IntegrationRule ir, DenseMatrix grad)
        GetGradients(GridFunction self, int const elem, IntegrationRule ir, DenseMatrix grad)
        """
        return _gridfunc.GridFunction_GetGradients(self, *args)
    GetGradients = _swig_new_instance_method(_gridfunc.GridFunction_GetGradients)

    def GetVectorGradient(self, tr, grad):
        r"""GetVectorGradient(GridFunction self, ElementTransformation tr, DenseMatrix grad)"""
        return _gridfunc.GridFunction_GetVectorGradient(self, tr, grad)
    GetVectorGradient = _swig_new_instance_method(_gridfunc.GridFunction_GetVectorGradient)

    def GetElementAverages(self, avgs):
        r"""GetElementAverages(GridFunction self, GridFunction avgs)"""
        return _gridfunc.GridFunction_GetElementAverages(self, avgs)
    GetElementAverages = _swig_new_instance_method(_gridfunc.GridFunction_GetElementAverages)

    def ImposeBounds(self, *args):
        r"""
        ImposeBounds(GridFunction self, int i, Vector weights, Vector _lo, Vector _hi)
        ImposeBounds(GridFunction self, int i, Vector weights, double _min=0.0, double _max=mfem::infinity())
        """
        return _gridfunc.GridFunction_ImposeBounds(self, *args)
    ImposeBounds = _swig_new_instance_method(_gridfunc.GridFunction_ImposeBounds)

    def RestrictConforming(self):
        r"""RestrictConforming(GridFunction self)"""
        return _gridfunc.GridFunction_RestrictConforming(self)
    RestrictConforming = _swig_new_instance_method(_gridfunc.GridFunction_RestrictConforming)

    def ProjectGridFunction(self, src):
        r"""ProjectGridFunction(GridFunction self, GridFunction src)"""
        return _gridfunc.GridFunction_ProjectGridFunction(self, src)
    ProjectGridFunction = _swig_new_instance_method(_gridfunc.GridFunction_ProjectGridFunction)

    def ProjectCoefficient(self, *args):
        r"""
        ProjectCoefficient(GridFunction self, Coefficient coeff)
        ProjectCoefficient(GridFunction self, Coefficient coeff, intArray dofs, int vd=0)
        ProjectCoefficient(GridFunction self, VectorCoefficient vcoeff)
        ProjectCoefficient(GridFunction self, VectorCoefficient vcoeff, intArray dofs)
        ProjectCoefficient(GridFunction self, mfem::Coefficient *[] coeff)
        """
        return _gridfunc.GridFunction_ProjectCoefficient(self, *args)
    ProjectCoefficient = _swig_new_instance_method(_gridfunc.GridFunction_ProjectCoefficient)
    ARITHMETIC = _gridfunc.GridFunction_ARITHMETIC
    
    HARMONIC = _gridfunc.GridFunction_HARMONIC
    

    def ProjectDiscCoefficient(self, *args):
        r"""
        ProjectDiscCoefficient(GridFunction self, VectorCoefficient coeff)
        ProjectDiscCoefficient(GridFunction self, Coefficient coeff, mfem::GridFunction::AvgType type)
        ProjectDiscCoefficient(GridFunction self, VectorCoefficient coeff, mfem::GridFunction::AvgType type)
        """
        return _gridfunc.GridFunction_ProjectDiscCoefficient(self, *args)
    ProjectDiscCoefficient = _swig_new_instance_method(_gridfunc.GridFunction_ProjectDiscCoefficient)

    def ProjectBdrCoefficient(self, *args):
        r"""
        ProjectBdrCoefficient(GridFunction self, Coefficient coeff, intArray attr)
        ProjectBdrCoefficient(GridFunction self, VectorCoefficient vcoeff, intArray attr)
        ProjectBdrCoefficient(GridFunction self, mfem::Coefficient *[] coeff, intArray attr)
        """
        return _gridfunc.GridFunction_ProjectBdrCoefficient(self, *args)
    ProjectBdrCoefficient = _swig_new_instance_method(_gridfunc.GridFunction_ProjectBdrCoefficient)

    def ProjectBdrCoefficientNormal(self, vcoeff, bdr_attr):
        r"""ProjectBdrCoefficientNormal(GridFunction self, VectorCoefficient vcoeff, intArray bdr_attr)"""
        return _gridfunc.GridFunction_ProjectBdrCoefficientNormal(self, vcoeff, bdr_attr)
    ProjectBdrCoefficientNormal = _swig_new_instance_method(_gridfunc.GridFunction_ProjectBdrCoefficientNormal)

    def ProjectBdrCoefficientTangent(self, vcoeff, bdr_attr):
        r"""ProjectBdrCoefficientTangent(GridFunction self, VectorCoefficient vcoeff, intArray bdr_attr)"""
        return _gridfunc.GridFunction_ProjectBdrCoefficientTangent(self, vcoeff, bdr_attr)
    ProjectBdrCoefficientTangent = _swig_new_instance_method(_gridfunc.GridFunction_ProjectBdrCoefficientTangent)

    def ComputeL2Error(self, *args):
        r"""
        ComputeL2Error(GridFunction self, Coefficient exsol, mfem::IntegrationRule const *[] irs=0) -> double
        ComputeL2Error(GridFunction self, mfem::Coefficient *[] exsol, mfem::IntegrationRule const *[] irs=0) -> double
        ComputeL2Error(GridFunction self, VectorCoefficient exsol, mfem::IntegrationRule const *[] irs=0, intArray elems=None) -> double
        """
        return _gridfunc.GridFunction_ComputeL2Error(self, *args)
    ComputeL2Error = _swig_new_instance_method(_gridfunc.GridFunction_ComputeL2Error)

    def ComputeGradError(self, exgrad, irs=0):
        r"""ComputeGradError(GridFunction self, VectorCoefficient exgrad, mfem::IntegrationRule const *[] irs=0) -> double"""
        return _gridfunc.GridFunction_ComputeGradError(self, exgrad, irs)
    ComputeGradError = _swig_new_instance_method(_gridfunc.GridFunction_ComputeGradError)

    def ComputeCurlError(self, excurl, irs=0):
        r"""ComputeCurlError(GridFunction self, VectorCoefficient excurl, mfem::IntegrationRule const *[] irs=0) -> double"""
        return _gridfunc.GridFunction_ComputeCurlError(self, excurl, irs)
    ComputeCurlError = _swig_new_instance_method(_gridfunc.GridFunction_ComputeCurlError)

    def ComputeDivError(self, exdiv, irs=0):
        r"""ComputeDivError(GridFunction self, Coefficient exdiv, mfem::IntegrationRule const *[] irs=0) -> double"""
        return _gridfunc.GridFunction_ComputeDivError(self, exdiv, irs)
    ComputeDivError = _swig_new_instance_method(_gridfunc.GridFunction_ComputeDivError)

    def ComputeDGFaceJumpError(self, exsol, ell_coeff, Nu, irs=0):
        r"""ComputeDGFaceJumpError(GridFunction self, Coefficient exsol, Coefficient ell_coeff, double Nu, mfem::IntegrationRule const *[] irs=0) -> double"""
        return _gridfunc.GridFunction_ComputeDGFaceJumpError(self, exsol, ell_coeff, Nu, irs)
    ComputeDGFaceJumpError = _swig_new_instance_method(_gridfunc.GridFunction_ComputeDGFaceJumpError)

    def ComputeH1Error(self, *args):
        r"""
        ComputeH1Error(GridFunction self, Coefficient exsol, VectorCoefficient exgrad, Coefficient ell_coef, double Nu, int norm_type) -> double
        ComputeH1Error(GridFunction self, Coefficient exsol, VectorCoefficient exgrad, mfem::IntegrationRule const *[] irs=0) -> double
        """
        return _gridfunc.GridFunction_ComputeH1Error(self, *args)
    ComputeH1Error = _swig_new_instance_method(_gridfunc.GridFunction_ComputeH1Error)

    def ComputeHDivError(self, exsol, exdiv, irs=0):
        r"""ComputeHDivError(GridFunction self, VectorCoefficient exsol, Coefficient exdiv, mfem::IntegrationRule const *[] irs=0) -> double"""
        return _gridfunc.GridFunction_ComputeHDivError(self, exsol, exdiv, irs)
    ComputeHDivError = _swig_new_instance_method(_gridfunc.GridFunction_ComputeHDivError)

    def ComputeHCurlError(self, exsol, excurl, irs=0):
        r"""ComputeHCurlError(GridFunction self, VectorCoefficient exsol, VectorCoefficient excurl, mfem::IntegrationRule const *[] irs=0) -> double"""
        return _gridfunc.GridFunction_ComputeHCurlError(self, exsol, excurl, irs)
    ComputeHCurlError = _swig_new_instance_method(_gridfunc.GridFunction_ComputeHCurlError)

    def ComputeMaxError(self, *args):
        r"""
        ComputeMaxError(GridFunction self, Coefficient exsol, mfem::IntegrationRule const *[] irs=0) -> double
        ComputeMaxError(GridFunction self, mfem::Coefficient *[] exsol, mfem::IntegrationRule const *[] irs=0) -> double
        ComputeMaxError(GridFunction self, VectorCoefficient exsol, mfem::IntegrationRule const *[] irs=0) -> double
        """
        return _gridfunc.GridFunction_ComputeMaxError(self, *args)
    ComputeMaxError = _swig_new_instance_method(_gridfunc.GridFunction_ComputeMaxError)

    def ComputeW11Error(self, exsol, exgrad, norm_type, elems=None, irs=0):
        r"""ComputeW11Error(GridFunction self, Coefficient exsol, VectorCoefficient exgrad, int norm_type, intArray elems=None, mfem::IntegrationRule const *[] irs=0) -> double"""
        return _gridfunc.GridFunction_ComputeW11Error(self, exsol, exgrad, norm_type, elems, irs)
    ComputeW11Error = _swig_new_instance_method(_gridfunc.GridFunction_ComputeW11Error)

    def ComputeL1Error(self, *args):
        r"""
        ComputeL1Error(GridFunction self, Coefficient exsol, mfem::IntegrationRule const *[] irs=0) -> double
        ComputeL1Error(GridFunction self, VectorCoefficient exsol, mfem::IntegrationRule const *[] irs=0) -> double
        """
        return _gridfunc.GridFunction_ComputeL1Error(self, *args)
    ComputeL1Error = _swig_new_instance_method(_gridfunc.GridFunction_ComputeL1Error)

    def ComputeLpError(self, *args):
        r"""
        ComputeLpError(GridFunction self, double const p, Coefficient exsol, Coefficient weight=None, mfem::IntegrationRule const *[] irs=0) -> double
        ComputeLpError(GridFunction self, double const p, VectorCoefficient exsol, Coefficient weight=None, VectorCoefficient v_weight=None, mfem::IntegrationRule const *[] irs=0) -> double
        """
        return _gridfunc.GridFunction_ComputeLpError(self, *args)
    ComputeLpError = _swig_new_instance_method(_gridfunc.GridFunction_ComputeLpError)

    def ComputeElementLpErrors(self, *args):
        r"""
        ComputeElementLpErrors(GridFunction self, double const p, Coefficient exsol, Vector error, Coefficient weight=None, mfem::IntegrationRule const *[] irs=0)
        ComputeElementLpErrors(GridFunction self, double const p, VectorCoefficient exsol, Vector error, Coefficient weight=None, VectorCoefficient v_weight=None, mfem::IntegrationRule const *[] irs=0)
        """
        return _gridfunc.GridFunction_ComputeElementLpErrors(self, *args)
    ComputeElementLpErrors = _swig_new_instance_method(_gridfunc.GridFunction_ComputeElementLpErrors)

    def ComputeElementL1Errors(self, *args):
        r"""
        ComputeElementL1Errors(GridFunction self, Coefficient exsol, Vector error, mfem::IntegrationRule const *[] irs=0)
        ComputeElementL1Errors(GridFunction self, VectorCoefficient exsol, Vector error, mfem::IntegrationRule const *[] irs=0)
        """
        return _gridfunc.GridFunction_ComputeElementL1Errors(self, *args)
    ComputeElementL1Errors = _swig_new_instance_method(_gridfunc.GridFunction_ComputeElementL1Errors)

    def ComputeElementL2Errors(self, *args):
        r"""
        ComputeElementL2Errors(GridFunction self, Coefficient exsol, Vector error, mfem::IntegrationRule const *[] irs=0)
        ComputeElementL2Errors(GridFunction self, VectorCoefficient exsol, Vector error, mfem::IntegrationRule const *[] irs=0)
        """
        return _gridfunc.GridFunction_ComputeElementL2Errors(self, *args)
    ComputeElementL2Errors = _swig_new_instance_method(_gridfunc.GridFunction_ComputeElementL2Errors)

    def ComputeElementMaxErrors(self, *args):
        r"""
        ComputeElementMaxErrors(GridFunction self, Coefficient exsol, Vector error, mfem::IntegrationRule const *[] irs=0)
        ComputeElementMaxErrors(GridFunction self, VectorCoefficient exsol, Vector error, mfem::IntegrationRule const *[] irs=0)
        """
        return _gridfunc.GridFunction_ComputeElementMaxErrors(self, *args)
    ComputeElementMaxErrors = _swig_new_instance_method(_gridfunc.GridFunction_ComputeElementMaxErrors)

    def ComputeFlux(self, blfi, flux, wcoef=True, subdomain=-1):
        r"""ComputeFlux(GridFunction self, BilinearFormIntegrator blfi, GridFunction flux, bool wcoef=True, int subdomain=-1)"""
        return _gridfunc.GridFunction_ComputeFlux(self, blfi, flux, wcoef, subdomain)
    ComputeFlux = _swig_new_instance_method(_gridfunc.GridFunction_ComputeFlux)

    def Assign(self, *args):
        r"""
        Assign(GridFunction self, GridFunction rhs) -> GridFunction
        Assign(GridFunction self, double value) -> GridFunction
        Assign(GridFunction self, Vector v) -> GridFunction
        """
        return _gridfunc.GridFunction_Assign(self, *args)
    Assign = _swig_new_instance_method(_gridfunc.GridFunction_Assign)

    def Update(self):
        r"""Update(GridFunction self)"""
        return _gridfunc.GridFunction_Update(self)
    Update = _swig_new_instance_method(_gridfunc.GridFunction_Update)

    def FESpace(self, *args):
        r"""
        FESpace(GridFunction self) -> FiniteElementSpace
        FESpace(GridFunction self) -> FiniteElementSpace
        """
        return _gridfunc.GridFunction_FESpace(self, *args)
    FESpace = _swig_new_instance_method(_gridfunc.GridFunction_FESpace)

    def SetSpace(self, f):
        r"""SetSpace(GridFunction self, FiniteElementSpace f)"""
        return _gridfunc.GridFunction_SetSpace(self, f)
    SetSpace = _swig_new_instance_method(_gridfunc.GridFunction_SetSpace)

    def MakeRef(self, *args):
        r"""
        MakeRef(GridFunction self, Vector base, int offset, int size)
        MakeRef(GridFunction self, Vector base, int offset)
        MakeRef(GridFunction self, FiniteElementSpace f, double * v)
        MakeRef(GridFunction self, FiniteElementSpace f, Vector v, int v_offset)
        """
        return _gridfunc.GridFunction_MakeRef(self, *args)
    MakeRef = _swig_new_instance_method(_gridfunc.GridFunction_MakeRef)

    def MakeTRef(self, *args):
        r"""
        MakeTRef(GridFunction self, FiniteElementSpace f, double * tv)
        MakeTRef(GridFunction self, FiniteElementSpace f, Vector tv, int tv_offset)
        """
        return _gridfunc.GridFunction_MakeTRef(self, *args)
    MakeTRef = _swig_new_instance_method(_gridfunc.GridFunction_MakeTRef)

    def SaveVTK(self, out, field_name, ref):
        r"""SaveVTK(GridFunction self, std::ostream & out, std::string const & field_name, int ref)"""
        return _gridfunc.GridFunction_SaveVTK(self, out, field_name, ref)
    SaveVTK = _swig_new_instance_method(_gridfunc.GridFunction_SaveVTK)

    def SaveSTL(self, out, TimesToRefine=1):
        r"""SaveSTL(GridFunction self, std::ostream & out, int TimesToRefine=1)"""
        return _gridfunc.GridFunction_SaveSTL(self, out, TimesToRefine)
    SaveSTL = _swig_new_instance_method(_gridfunc.GridFunction_SaveSTL)
    __swig_destroy__ = _gridfunc.delete_GridFunction

    def __init__(self, *args):
        r"""
        __init__(GridFunction self) -> GridFunction
        __init__(GridFunction self, GridFunction orig) -> GridFunction
        __init__(GridFunction self, FiniteElementSpace f) -> GridFunction
        __init__(GridFunction self, FiniteElementSpace f, double * data) -> GridFunction
        __init__(GridFunction self, Mesh m, std::istream & input) -> GridFunction
        __init__(GridFunction self, Mesh m, mfem::GridFunction *[] gf_array, int num_pieces) -> GridFunction
        __init__(GridFunction self, Mesh m, char const * grid_file) -> GridFunction
        __init__(GridFunction self, FiniteElementSpace fes, Vector v, int offset) -> GridFunction
        """
        _gridfunc.GridFunction_swiginit(self, _gridfunc.new_GridFunction(*args))

    def SaveToFile(self, gf_file, precision):
        r"""SaveToFile(GridFunction self, char const * gf_file, int const precision)"""
        return _gridfunc.GridFunction_SaveToFile(self, gf_file, precision)
    SaveToFile = _swig_new_instance_method(_gridfunc.GridFunction_SaveToFile)

    def WriteToStream(self, StringIO):
        r"""WriteToStream(GridFunction self, PyObject * StringIO) -> PyObject *"""
        return _gridfunc.GridFunction_WriteToStream(self, StringIO)
    WriteToStream = _swig_new_instance_method(_gridfunc.GridFunction_WriteToStream)

    def iadd(self, c):
        r"""iadd(GridFunction self, GridFunction c) -> GridFunction"""
        return _gridfunc.GridFunction_iadd(self, c)
    iadd = _swig_new_instance_method(_gridfunc.GridFunction_iadd)

    def isub(self, *args):
        r"""
        isub(GridFunction self, GridFunction c) -> GridFunction
        isub(GridFunction self, double c) -> GridFunction
        """
        return _gridfunc.GridFunction_isub(self, *args)
    isub = _swig_new_instance_method(_gridfunc.GridFunction_isub)

    def imul(self, c):
        r"""imul(GridFunction self, double c) -> GridFunction"""
        return _gridfunc.GridFunction_imul(self, c)
    imul = _swig_new_instance_method(_gridfunc.GridFunction_imul)

    def idiv(self, c):
        r"""idiv(GridFunction self, double c) -> GridFunction"""
        return _gridfunc.GridFunction_idiv(self, c)
    idiv = _swig_new_instance_method(_gridfunc.GridFunction_idiv)

    def Save(self, *args):
        r"""
        Save(GridFunction self, std::ostream & out)
        Save(GridFunction self, char const * file, int precision=8)
        """
        return _gridfunc.GridFunction_Save(self, *args)
    Save = _swig_new_instance_method(_gridfunc.GridFunction_Save)

# Register GridFunction in _gridfunc:
_gridfunc.GridFunction_swigregister(GridFunction)

class QuadratureFunction(mfem._par.vector.Vector):
    r"""Proxy of C++ mfem::QuadratureFunction class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(QuadratureFunction self) -> QuadratureFunction
        __init__(QuadratureFunction self, QuadratureFunction orig) -> QuadratureFunction
        __init__(QuadratureFunction self, QuadratureSpace qspace_, int vdim_=1) -> QuadratureFunction
        __init__(QuadratureFunction self, QuadratureSpace qspace_, double * qf_data, int vdim_=1) -> QuadratureFunction
        __init__(QuadratureFunction self, Mesh mesh, std::istream & _in) -> QuadratureFunction
        """
        _gridfunc.QuadratureFunction_swiginit(self, _gridfunc.new_QuadratureFunction(*args))
    __swig_destroy__ = _gridfunc.delete_QuadratureFunction

    def GetSpace(self):
        r"""GetSpace(QuadratureFunction self) -> QuadratureSpace"""
        return _gridfunc.QuadratureFunction_GetSpace(self)
    GetSpace = _swig_new_instance_method(_gridfunc.QuadratureFunction_GetSpace)

    def SetSpace(self, *args):
        r"""
        SetSpace(QuadratureFunction self, QuadratureSpace qspace_, int vdim_=-1)
        SetSpace(QuadratureFunction self, QuadratureSpace qspace_, double * qf_data, int vdim_=-1)
        """
        return _gridfunc.QuadratureFunction_SetSpace(self, *args)
    SetSpace = _swig_new_instance_method(_gridfunc.QuadratureFunction_SetSpace)

    def GetVDim(self):
        r"""GetVDim(QuadratureFunction self) -> int"""
        return _gridfunc.QuadratureFunction_GetVDim(self)
    GetVDim = _swig_new_instance_method(_gridfunc.QuadratureFunction_GetVDim)

    def SetVDim(self, vdim_):
        r"""SetVDim(QuadratureFunction self, int vdim_)"""
        return _gridfunc.QuadratureFunction_SetVDim(self, vdim_)
    SetVDim = _swig_new_instance_method(_gridfunc.QuadratureFunction_SetVDim)

    def OwnsSpace(self):
        r"""OwnsSpace(QuadratureFunction self) -> bool"""
        return _gridfunc.QuadratureFunction_OwnsSpace(self)
    OwnsSpace = _swig_new_instance_method(_gridfunc.QuadratureFunction_OwnsSpace)

    def SetOwnsSpace(self, own):
        r"""SetOwnsSpace(QuadratureFunction self, bool own)"""
        return _gridfunc.QuadratureFunction_SetOwnsSpace(self, own)
    SetOwnsSpace = _swig_new_instance_method(_gridfunc.QuadratureFunction_SetOwnsSpace)

    def GetElementIntRule(self, idx):
        r"""GetElementIntRule(QuadratureFunction self, int idx) -> IntegrationRule"""
        return _gridfunc.QuadratureFunction_GetElementIntRule(self, idx)
    GetElementIntRule = _swig_new_instance_method(_gridfunc.QuadratureFunction_GetElementIntRule)

    def GetElementValues(self, *args):
        r"""
        GetElementValues(QuadratureFunction self, int idx, Vector values)
        GetElementValues(QuadratureFunction self, int idx, Vector values)
        GetElementValues(QuadratureFunction self, int idx, int const ip_num, Vector values)
        GetElementValues(QuadratureFunction self, int idx, int const ip_num, Vector values)
        GetElementValues(QuadratureFunction self, int idx, DenseMatrix values)
        GetElementValues(QuadratureFunction self, int idx, DenseMatrix values)
        """
        return _gridfunc.QuadratureFunction_GetElementValues(self, *args)
    GetElementValues = _swig_new_instance_method(_gridfunc.QuadratureFunction_GetElementValues)

    def Save(self, *args):
        r"""
        Save(QuadratureFunction self, std::ostream & out)
        Save(QuadratureFunction self, char const * file, int precision=8)
        """
        return _gridfunc.QuadratureFunction_Save(self, *args)
    Save = _swig_new_instance_method(_gridfunc.QuadratureFunction_Save)

# Register QuadratureFunction in _gridfunc:
_gridfunc.QuadratureFunction_swigregister(QuadratureFunction)


def __lshift__(*args):
    r"""
    __lshift__(std::ostream & os, SparseMatrix mat) -> std::ostream
    __lshift__(std::ostream & out, Mesh mesh) -> std::ostream
    __lshift__(std::ostream & out, GridFunction sol) -> std::ostream
    __lshift__(std::ostream & out, QuadratureFunction qf) -> std::ostream &
    """
    return _gridfunc.__lshift__(*args)
__lshift__ = _gridfunc.__lshift__

def ZZErrorEstimator(blfi, u, flux, error_estimates, aniso_flags=None, with_subdomains=1, with_coeff=False):
    r"""ZZErrorEstimator(BilinearFormIntegrator blfi, GridFunction u, GridFunction flux, Vector error_estimates, intArray aniso_flags=None, int with_subdomains=1, bool with_coeff=False) -> double"""
    return _gridfunc.ZZErrorEstimator(blfi, u, flux, error_estimates, aniso_flags, with_subdomains, with_coeff)
ZZErrorEstimator = _gridfunc.ZZErrorEstimator

def ComputeElementLpDistance(p, i, gf1, gf2):
    r"""ComputeElementLpDistance(double p, int i, GridFunction gf1, GridFunction gf2) -> double"""
    return _gridfunc.ComputeElementLpDistance(p, i, gf1, gf2)
ComputeElementLpDistance = _gridfunc.ComputeElementLpDistance
class ExtrudeCoefficient(mfem._par.coefficient.Coefficient):
    r"""Proxy of C++ mfem::ExtrudeCoefficient class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, m, s, _n):
        r"""__init__(ExtrudeCoefficient self, Mesh m, Coefficient s, int _n) -> ExtrudeCoefficient"""
        _gridfunc.ExtrudeCoefficient_swiginit(self, _gridfunc.new_ExtrudeCoefficient(m, s, _n))

    def Eval(self, T, ip):
        r"""Eval(ExtrudeCoefficient self, ElementTransformation T, IntegrationPoint ip) -> double"""
        return _gridfunc.ExtrudeCoefficient_Eval(self, T, ip)
    Eval = _swig_new_instance_method(_gridfunc.ExtrudeCoefficient_Eval)
    __swig_destroy__ = _gridfunc.delete_ExtrudeCoefficient

# Register ExtrudeCoefficient in _gridfunc:
_gridfunc.ExtrudeCoefficient_swigregister(ExtrudeCoefficient)


def Extrude1DGridFunction(mesh, mesh2d, sol, ny):
    r"""Extrude1DGridFunction(Mesh mesh, Mesh mesh2d, GridFunction sol, int const ny) -> GridFunction"""
    return _gridfunc.Extrude1DGridFunction(mesh, mesh2d, sol, ny)
Extrude1DGridFunction = _gridfunc.Extrude1DGridFunction

def __iadd__(self, v):
    ret = _gridfunc.GridFunction_iadd(self, v)
    ret.thisown = 0
    return self
def __isub__(self, v):
    ret = _gridfunc.GridFunction_isub(self, v)
    ret.thisown = 0
    return self
def __idiv__(self, v):
    ret = _gridfunc.GridFunction_idiv(self, v)
    ret.thisown = 0
    return self
def __imul__(self, v):
    ret = _gridfunc.GridFunction_imul(self, v)
    ret.thisown = 0
    return self

GridFunction.__iadd__  = __iadd__
GridFunction.__idiv__  = __idiv__
GridFunction.__isub__  = __isub__
GridFunction.__imul__  = __imul__      






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.8
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.





from sys import version_info
if version_info >= (2, 6, 0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_fem', [dirname(__file__)])
        except ImportError:
            import _fem
            return _fem
        if fp is not None:
            try:
                _mod = imp.load_module('_fem', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _fem = swig_import_helper()
    del swig_import_helper
else:
    import _fem
del version_info
try:
    _swig_property = property
except NameError:
    pass  # Python < 2.2 doesn't have 'property'.


def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"):
        return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method:
        return method(self, value)
    if (not static):
        if _newclass:
            object.__setattr__(self, name, value)
        else:
            self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)


def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)


def _swig_getattr_nondynamic(self, class_type, name, static=1):
    if (name == "thisown"):
        return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method:
        return method(self)
    if (not static):
        return object.__getattr__(self, name)
    else:
        raise AttributeError(name)

def _swig_getattr(self, class_type, name):
    return _swig_getattr_nondynamic(self, class_type, name, 0)


def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object:
        pass
    _newclass = 0


class Ordering(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Ordering, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Ordering, name)
    __repr__ = _swig_repr
    byNODES = _fem.Ordering_byNODES
    byVDIM = _fem.Ordering_byVDIM

    def __init__(self):
        this = _fem.new_Ordering()
        try:
            self.this.append(this)
        except Exception:
            self.this = this
    __swig_destroy__ = _fem.delete_Ordering
    __del__ = lambda self: None
Ordering_swigregister = _fem.Ordering_swigregister
Ordering_swigregister(Ordering)

class RefinementData(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, RefinementData, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, RefinementData, name)
    __repr__ = _swig_repr
    __swig_setmethods__["type"] = _fem.RefinementData_type_set
    __swig_getmethods__["type"] = _fem.RefinementData_type_get
    if _newclass:
        type = _swig_property(_fem.RefinementData_type_get, _fem.RefinementData_type_set)
    __swig_setmethods__["num_fine_elems"] = _fem.RefinementData_num_fine_elems_set
    __swig_getmethods__["num_fine_elems"] = _fem.RefinementData_num_fine_elems_get
    if _newclass:
        num_fine_elems = _swig_property(_fem.RefinementData_num_fine_elems_get, _fem.RefinementData_num_fine_elems_set)
    __swig_setmethods__["num_fine_dofs"] = _fem.RefinementData_num_fine_dofs_set
    __swig_getmethods__["num_fine_dofs"] = _fem.RefinementData_num_fine_dofs_get
    if _newclass:
        num_fine_dofs = _swig_property(_fem.RefinementData_num_fine_dofs_get, _fem.RefinementData_num_fine_dofs_set)
    __swig_setmethods__["fl_to_fc"] = _fem.RefinementData_fl_to_fc_set
    __swig_getmethods__["fl_to_fc"] = _fem.RefinementData_fl_to_fc_get
    if _newclass:
        fl_to_fc = _swig_property(_fem.RefinementData_fl_to_fc_get, _fem.RefinementData_fl_to_fc_set)
    __swig_setmethods__["I"] = _fem.RefinementData_I_set
    __swig_getmethods__["I"] = _fem.RefinementData_I_get
    if _newclass:
        I = _swig_property(_fem.RefinementData_I_get, _fem.RefinementData_I_set)
    __swig_destroy__ = _fem.delete_RefinementData
    __del__ = lambda self: None

    def __init__(self):
        this = _fem.new_RefinementData()
        try:
            self.this.append(this)
        except Exception:
            self.this = this
RefinementData_swigregister = _fem.RefinementData_swigregister
RefinementData_swigregister(RefinementData)

class FiniteElementSpace(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, FiniteElementSpace, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, FiniteElementSpace, name)
    __repr__ = _swig_repr

    def __init__(self, *args):
        this = _fem.new_FiniteElementSpace(*args)
        try:
            self.this.append(this)
        except Exception:
            self.this = this

    def GetMesh(self):
        return _fem.FiniteElementSpace_GetMesh(self)

    def GetNURBSext(self):
        return _fem.FiniteElementSpace_GetNURBSext(self)

    def StealNURBSext(self):
        return _fem.FiniteElementSpace_StealNURBSext(self)

    def Conforming(self):
        return _fem.FiniteElementSpace_Conforming(self)

    def Nonconforming(self):
        return _fem.FiniteElementSpace_Nonconforming(self)

    def GetConformingProlongation(self):
        return _fem.FiniteElementSpace_GetConformingProlongation(self)

    def GetConformingRestriction(self):
        return _fem.FiniteElementSpace_GetConformingRestriction(self)

    def GetRestrictionMatrix(self):
        return _fem.FiniteElementSpace_GetRestrictionMatrix(self)

    def GetVDim(self):
        return _fem.FiniteElementSpace_GetVDim(self)

    def GetOrder(self, i):
        return _fem.FiniteElementSpace_GetOrder(self, i)

    def GetFaceOrder(self, i):
        return _fem.FiniteElementSpace_GetFaceOrder(self, i)

    def GetNDofs(self):
        return _fem.FiniteElementSpace_GetNDofs(self)

    def GetVSize(self):
        return _fem.FiniteElementSpace_GetVSize(self)

    def GetTrueVSize(self):
        return _fem.FiniteElementSpace_GetTrueVSize(self)

    def GetNConformingDofs(self):
        return _fem.FiniteElementSpace_GetNConformingDofs(self)

    def GetConformingVSize(self):
        return _fem.FiniteElementSpace_GetConformingVSize(self)

    def GetOrdering(self):
        return _fem.FiniteElementSpace_GetOrdering(self)

    def FEColl(self):
        return _fem.FiniteElementSpace_FEColl(self)

    def GetNVDofs(self):
        return _fem.FiniteElementSpace_GetNVDofs(self)

    def GetNEDofs(self):
        return _fem.FiniteElementSpace_GetNEDofs(self)

    def GetNFDofs(self):
        return _fem.FiniteElementSpace_GetNFDofs(self)

    def GetNE(self):
        return _fem.FiniteElementSpace_GetNE(self)

    def GetNV(self):
        return _fem.FiniteElementSpace_GetNV(self)

    def GetNBE(self):
        return _fem.FiniteElementSpace_GetNBE(self)

    def GetElementType(self, i):
        return _fem.FiniteElementSpace_GetElementType(self, i)

    def GetElementVertices(self, i, vertices):
        return _fem.FiniteElementSpace_GetElementVertices(self, i, vertices)

    def GetBdrElementType(self, i):
        return _fem.FiniteElementSpace_GetBdrElementType(self, i)

    def GetElementTransformation(self, *args):
        return _fem.FiniteElementSpace_GetElementTransformation(self, *args)

    def GetBdrElementTransformation(self, i):
        return _fem.FiniteElementSpace_GetBdrElementTransformation(self, i)

    def GetAttribute(self, i):
        return _fem.FiniteElementSpace_GetAttribute(self, i)

    def GetBdrAttribute(self, i):
        return _fem.FiniteElementSpace_GetBdrAttribute(self, i)

    def GetElementDofs(self, i, dofs):
        return _fem.FiniteElementSpace_GetElementDofs(self, i, dofs)

    def GetBdrElementDofs(self, i, dofs):
        return _fem.FiniteElementSpace_GetBdrElementDofs(self, i, dofs)

    def GetFaceDofs(self, i, dofs):
        return _fem.FiniteElementSpace_GetFaceDofs(self, i, dofs)

    def GetEdgeDofs(self, i, dofs):
        return _fem.FiniteElementSpace_GetEdgeDofs(self, i, dofs)

    def GetVertexDofs(self, i, dofs):
        return _fem.FiniteElementSpace_GetVertexDofs(self, i, dofs)

    def GetElementInteriorDofs(self, i, dofs):
        return _fem.FiniteElementSpace_GetElementInteriorDofs(self, i, dofs)

    def GetNumElementInteriorDofs(self, i):
        return _fem.FiniteElementSpace_GetNumElementInteriorDofs(self, i)

    def GetEdgeInteriorDofs(self, i, dofs):
        return _fem.FiniteElementSpace_GetEdgeInteriorDofs(self, i, dofs)

    def DofsToVDofs(self, *args):
        return _fem.FiniteElementSpace_DofsToVDofs(self, *args)

    def DofToVDof(self, dof, vd, ndofs=-1):
        return _fem.FiniteElementSpace_DofToVDof(self, dof, vd, ndofs)

    def VDofToDof(self, vdof):
        return _fem.FiniteElementSpace_VDofToDof(self, vdof)
    __swig_getmethods__["AdjustVDofs"] = lambda x: _fem.FiniteElementSpace_AdjustVDofs
    if _newclass:
        AdjustVDofs = staticmethod(_fem.FiniteElementSpace_AdjustVDofs)

    def GetElementVDofs(self, i, vdofs):
        return _fem.FiniteElementSpace_GetElementVDofs(self, i, vdofs)

    def GetBdrElementVDofs(self, i, vdofs):
        return _fem.FiniteElementSpace_GetBdrElementVDofs(self, i, vdofs)

    def GetFaceVDofs(self, i, vdofs):
        return _fem.FiniteElementSpace_GetFaceVDofs(self, i, vdofs)

    def GetEdgeVDofs(self, i, vdofs):
        return _fem.FiniteElementSpace_GetEdgeVDofs(self, i, vdofs)

    def GetVertexVDofs(self, i, vdofs):
        return _fem.FiniteElementSpace_GetVertexVDofs(self, i, vdofs)

    def GetElementInteriorVDofs(self, i, vdofs):
        return _fem.FiniteElementSpace_GetElementInteriorVDofs(self, i, vdofs)

    def GetEdgeInteriorVDofs(self, i, vdofs):
        return _fem.FiniteElementSpace_GetEdgeInteriorVDofs(self, i, vdofs)

    def BuildElementToDofTable(self):
        return _fem.FiniteElementSpace_BuildElementToDofTable(self)

    def BuildDofToArrays(self):
        return _fem.FiniteElementSpace_BuildDofToArrays(self)

    def GetElementToDofTable(self):
        return _fem.FiniteElementSpace_GetElementToDofTable(self)

    def GetBdrElementToDofTable(self):
        return _fem.FiniteElementSpace_GetBdrElementToDofTable(self)

    def GetElementForDof(self, i):
        return _fem.FiniteElementSpace_GetElementForDof(self, i)

    def GetLocalDofForDof(self, i):
        return _fem.FiniteElementSpace_GetLocalDofForDof(self, i)

    def GetFE(self, i):
        return _fem.FiniteElementSpace_GetFE(self, i)

    def GetBE(self, i):
        return _fem.FiniteElementSpace_GetBE(self, i)

    def GetFaceElement(self, i):
        return _fem.FiniteElementSpace_GetFaceElement(self, i)

    def GetEdgeElement(self, i):
        return _fem.FiniteElementSpace_GetEdgeElement(self, i)

    def GetTraceElement(self, i, geom_type):
        return _fem.FiniteElementSpace_GetTraceElement(self, i, geom_type)

    def GetEssentialVDofs(self, bdr_attr_is_ess, ess_vdofs):
        return _fem.FiniteElementSpace_GetEssentialVDofs(self, bdr_attr_is_ess, ess_vdofs)

    def GetEssentialTrueDofs(self, bdr_attr_is_ess, ess_tdof_list):
        return _fem.FiniteElementSpace_GetEssentialTrueDofs(self, bdr_attr_is_ess, ess_tdof_list)
    __swig_getmethods__["MarkerToList"] = lambda x: _fem.FiniteElementSpace_MarkerToList
    if _newclass:
        MarkerToList = staticmethod(_fem.FiniteElementSpace_MarkerToList)
    __swig_getmethods__["ListToMarker"] = lambda x: _fem.FiniteElementSpace_ListToMarker
    if _newclass:
        ListToMarker = staticmethod(_fem.FiniteElementSpace_ListToMarker)

    def ConvertToConformingVDofs(self, dofs, cdofs):
        return _fem.FiniteElementSpace_ConvertToConformingVDofs(self, dofs, cdofs)

    def ConvertFromConformingVDofs(self, cdofs, dofs):
        return _fem.FiniteElementSpace_ConvertFromConformingVDofs(self, cdofs, dofs)

    def EliminateEssentialBCFromGRM(self, cfes, bdr_attr_is_ess, R):
        return _fem.FiniteElementSpace_EliminateEssentialBCFromGRM(self, cfes, bdr_attr_is_ess, R)

    def GlobalRestrictionMatrix(self, *args):
        return _fem.FiniteElementSpace_GlobalRestrictionMatrix(self, *args)

    def D2C_GlobalRestrictionMatrix(self, cfes):
        return _fem.FiniteElementSpace_D2C_GlobalRestrictionMatrix(self, cfes)

    def D2Const_GlobalRestrictionMatrix(self, cfes):
        return _fem.FiniteElementSpace_D2Const_GlobalRestrictionMatrix(self, cfes)

    def H2L_GlobalRestrictionMatrix(self, lfes):
        return _fem.FiniteElementSpace_H2L_GlobalRestrictionMatrix(self, lfes)

    def Update(self):
        return _fem.FiniteElementSpace_Update(self)

    def UpdateAndInterpolate(self, *args):
        return _fem.FiniteElementSpace_UpdateAndInterpolate(self, *args)

    def SaveUpdate(self):
        return _fem.FiniteElementSpace_SaveUpdate(self)

    def Save(self, out):
        return _fem.FiniteElementSpace_Save(self, out)
    __swig_destroy__ = _fem.delete_FiniteElementSpace
    __del__ = lambda self: None
FiniteElementSpace_swigregister = _fem.FiniteElementSpace_swigregister
FiniteElementSpace_swigregister(FiniteElementSpace)

def FiniteElementSpace_AdjustVDofs(vdofs):
    return _fem.FiniteElementSpace_AdjustVDofs(vdofs)
FiniteElementSpace_AdjustVDofs = _fem.FiniteElementSpace_AdjustVDofs

def FiniteElementSpace_MarkerToList(marker, list):
    return _fem.FiniteElementSpace_MarkerToList(marker, list)
FiniteElementSpace_MarkerToList = _fem.FiniteElementSpace_MarkerToList

def FiniteElementSpace_ListToMarker(list, marker_size, marker, mark_val=-1):
    return _fem.FiniteElementSpace_ListToMarker(list, marker_size, marker, mark_val)
FiniteElementSpace_ListToMarker = _fem.FiniteElementSpace_ListToMarker

# This file is compatible with both classic and new-style classes.





# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _restriction
else:
    import _restriction

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _restriction.SWIG_PyInstanceMethod_New
_swig_new_static_method = _restriction.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.element
import mfem._par.globals
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.densemat
import mfem._par.vector
import mfem._par.operators
import mfem._par.matrix
import mfem._par.geom
import mfem._par.intrules
import mfem._par.table
import mfem._par.hash
import mfem._par.mesh
import mfem._par.sort_pairs
import mfem._par.ncmesh
import mfem._par.vtk
import mfem._par.vertex
import mfem._par.gridfunc
import mfem._par.coefficient
import mfem._par.sparsemat
import mfem._par.eltrans
import mfem._par.fe
import mfem._par.fespace
import mfem._par.fe_coll
import mfem._par.lininteg
import mfem._par.handle
import mfem._par.hypre
import mfem._par.bilininteg
import mfem._par.linearform
L2FaceValues_SingleValued = _restriction.L2FaceValues_SingleValued

L2FaceValues_DoubleValued = _restriction.L2FaceValues_DoubleValued

class ElementRestriction(mfem._par.operators.Operator):
    r"""Proxy of C++ mfem::ElementRestriction class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, arg2, arg3):
        r"""__init__(ElementRestriction self, FiniteElementSpace arg2, mfem::ElementDofOrdering arg3) -> ElementRestriction"""
        _restriction.ElementRestriction_swiginit(self, _restriction.new_ElementRestriction(arg2, arg3))

    def Mult(self, x, y):
        r"""Mult(ElementRestriction self, Vector x, Vector y)"""
        return _restriction.ElementRestriction_Mult(self, x, y)
    Mult = _swig_new_instance_method(_restriction.ElementRestriction_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(ElementRestriction self, Vector x, Vector y)"""
        return _restriction.ElementRestriction_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_restriction.ElementRestriction_MultTranspose)

    def MultUnsigned(self, x, y):
        r"""MultUnsigned(ElementRestriction self, Vector x, Vector y)"""
        return _restriction.ElementRestriction_MultUnsigned(self, x, y)
    MultUnsigned = _swig_new_instance_method(_restriction.ElementRestriction_MultUnsigned)

    def MultTransposeUnsigned(self, x, y):
        r"""MultTransposeUnsigned(ElementRestriction self, Vector x, Vector y)"""
        return _restriction.ElementRestriction_MultTransposeUnsigned(self, x, y)
    MultTransposeUnsigned = _swig_new_instance_method(_restriction.ElementRestriction_MultTransposeUnsigned)

    def BooleanMask(self, y):
        r"""BooleanMask(ElementRestriction self, Vector y)"""
        return _restriction.ElementRestriction_BooleanMask(self, y)
    BooleanMask = _swig_new_instance_method(_restriction.ElementRestriction_BooleanMask)

    def FillSparseMatrix(self, mat_ea, mat):
        r"""FillSparseMatrix(ElementRestriction self, Vector mat_ea, SparseMatrix mat)"""
        return _restriction.ElementRestriction_FillSparseMatrix(self, mat_ea, mat)
    FillSparseMatrix = _swig_new_instance_method(_restriction.ElementRestriction_FillSparseMatrix)

    def FillI(self, mat):
        r"""FillI(ElementRestriction self, SparseMatrix mat) -> int"""
        return _restriction.ElementRestriction_FillI(self, mat)
    FillI = _swig_new_instance_method(_restriction.ElementRestriction_FillI)

    def FillJAndData(self, ea_data, mat):
        r"""FillJAndData(ElementRestriction self, Vector ea_data, SparseMatrix mat)"""
        return _restriction.ElementRestriction_FillJAndData(self, ea_data, mat)
    FillJAndData = _swig_new_instance_method(_restriction.ElementRestriction_FillJAndData)
    __swig_destroy__ = _restriction.delete_ElementRestriction

# Register ElementRestriction in _restriction:
_restriction.ElementRestriction_swigregister(ElementRestriction)

class L2ElementRestriction(mfem._par.operators.Operator):
    r"""Proxy of C++ mfem::L2ElementRestriction class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, arg2):
        r"""__init__(L2ElementRestriction self, FiniteElementSpace arg2) -> L2ElementRestriction"""
        _restriction.L2ElementRestriction_swiginit(self, _restriction.new_L2ElementRestriction(arg2))

    def Mult(self, x, y):
        r"""Mult(L2ElementRestriction self, Vector x, Vector y)"""
        return _restriction.L2ElementRestriction_Mult(self, x, y)
    Mult = _swig_new_instance_method(_restriction.L2ElementRestriction_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(L2ElementRestriction self, Vector x, Vector y)"""
        return _restriction.L2ElementRestriction_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_restriction.L2ElementRestriction_MultTranspose)

    def FillI(self, mat):
        r"""FillI(L2ElementRestriction self, SparseMatrix mat)"""
        return _restriction.L2ElementRestriction_FillI(self, mat)
    FillI = _swig_new_instance_method(_restriction.L2ElementRestriction_FillI)

    def FillJAndData(self, ea_data, mat):
        r"""FillJAndData(L2ElementRestriction self, Vector ea_data, SparseMatrix mat)"""
        return _restriction.L2ElementRestriction_FillJAndData(self, ea_data, mat)
    FillJAndData = _swig_new_instance_method(_restriction.L2ElementRestriction_FillJAndData)
    __swig_destroy__ = _restriction.delete_L2ElementRestriction

# Register L2ElementRestriction in _restriction:
_restriction.L2ElementRestriction_swigregister(L2ElementRestriction)

class H1FaceRestriction(mfem._par.operators.Operator):
    r"""Proxy of C++ mfem::H1FaceRestriction class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, arg2, arg3, arg4):
        r"""__init__(H1FaceRestriction self, FiniteElementSpace arg2, mfem::ElementDofOrdering const arg3, mfem::FaceType const arg4) -> H1FaceRestriction"""
        _restriction.H1FaceRestriction_swiginit(self, _restriction.new_H1FaceRestriction(arg2, arg3, arg4))

    def Mult(self, x, y):
        r"""Mult(H1FaceRestriction self, Vector x, Vector y)"""
        return _restriction.H1FaceRestriction_Mult(self, x, y)
    Mult = _swig_new_instance_method(_restriction.H1FaceRestriction_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(H1FaceRestriction self, Vector x, Vector y)"""
        return _restriction.H1FaceRestriction_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_restriction.H1FaceRestriction_MultTranspose)
    __swig_destroy__ = _restriction.delete_H1FaceRestriction

# Register H1FaceRestriction in _restriction:
_restriction.H1FaceRestriction_swigregister(H1FaceRestriction)

class L2FaceRestriction(mfem._par.operators.Operator):
    r"""Proxy of C++ mfem::L2FaceRestriction class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""__init__(L2FaceRestriction self, FiniteElementSpace arg2, mfem::ElementDofOrdering const arg3, mfem::FaceType const arg4, mfem::L2FaceValues const m=DoubleValued) -> L2FaceRestriction"""
        _restriction.L2FaceRestriction_swiginit(self, _restriction.new_L2FaceRestriction(*args))

    def Mult(self, x, y):
        r"""Mult(L2FaceRestriction self, Vector x, Vector y)"""
        return _restriction.L2FaceRestriction_Mult(self, x, y)
    Mult = _swig_new_instance_method(_restriction.L2FaceRestriction_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(L2FaceRestriction self, Vector x, Vector y)"""
        return _restriction.L2FaceRestriction_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_restriction.L2FaceRestriction_MultTranspose)

    def FillI(self, mat, face_mat):
        r"""FillI(L2FaceRestriction self, SparseMatrix mat, SparseMatrix face_mat)"""
        return _restriction.L2FaceRestriction_FillI(self, mat, face_mat)
    FillI = _swig_new_instance_method(_restriction.L2FaceRestriction_FillI)

    def FillJAndData(self, ea_data, mat, face_mat):
        r"""FillJAndData(L2FaceRestriction self, Vector ea_data, SparseMatrix mat, SparseMatrix face_mat)"""
        return _restriction.L2FaceRestriction_FillJAndData(self, ea_data, mat, face_mat)
    FillJAndData = _swig_new_instance_method(_restriction.L2FaceRestriction_FillJAndData)

    def AddFaceMatricesToElementMatrices(self, fea_data, ea_data):
        r"""AddFaceMatricesToElementMatrices(L2FaceRestriction self, Vector fea_data, Vector ea_data)"""
        return _restriction.L2FaceRestriction_AddFaceMatricesToElementMatrices(self, fea_data, ea_data)
    AddFaceMatricesToElementMatrices = _swig_new_instance_method(_restriction.L2FaceRestriction_AddFaceMatricesToElementMatrices)
    __swig_destroy__ = _restriction.delete_L2FaceRestriction

# Register L2FaceRestriction in _restriction:
_restriction.L2FaceRestriction_swigregister(L2FaceRestriction)


def GetFaceDofs(dim, face_id, dof1d, faceMap):
    r"""GetFaceDofs(int const dim, int const face_id, int const dof1d, intArray faceMap)"""
    return _restriction.GetFaceDofs(dim, face_id, dof1d, faceMap)
GetFaceDofs = _restriction.GetFaceDofs

def ToLexOrdering(dim, face_id, size1d, index):
    r"""ToLexOrdering(int const dim, int const face_id, int const size1d, int const index) -> int"""
    return _restriction.ToLexOrdering(dim, face_id, size1d, index)
ToLexOrdering = _restriction.ToLexOrdering

def PermuteFaceL2(dim, face_id1, face_id2, orientation, size1d, index):
    r"""PermuteFaceL2(int const dim, int const face_id1, int const face_id2, int const orientation, int const size1d, int const index) -> int"""
    return _restriction.PermuteFaceL2(dim, face_id1, face_id2, orientation, size1d, index)
PermuteFaceL2 = _restriction.PermuteFaceL2





# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _stable3d
else:
    import _stable3d

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _stable3d.SWIG_PyInstanceMethod_New
_swig_new_static_method = _stable3d.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.element
import mfem._par.globals
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.densemat
import mfem._par.vector
import mfem._par.operators
import mfem._par.matrix
import mfem._par.geom
import mfem._par.intrules
import mfem._par.table
import mfem._par.hash
class STable3DNode(object):
    r"""Proxy of C++ mfem::STable3DNode class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    Prev = property(_stable3d.STable3DNode_Prev_get, _stable3d.STable3DNode_Prev_set, doc=r"""Prev : p.mfem::STable3DNode""")
    Column = property(_stable3d.STable3DNode_Column_get, _stable3d.STable3DNode_Column_set, doc=r"""Column : int""")
    Floor = property(_stable3d.STable3DNode_Floor_get, _stable3d.STable3DNode_Floor_set, doc=r"""Floor : int""")
    Number = property(_stable3d.STable3DNode_Number_get, _stable3d.STable3DNode_Number_set, doc=r"""Number : int""")

    def __init__(self):
        r"""__init__(STable3DNode self) -> STable3DNode"""
        _stable3d.STable3DNode_swiginit(self, _stable3d.new_STable3DNode())
    __swig_destroy__ = _stable3d.delete_STable3DNode

# Register STable3DNode in _stable3d:
_stable3d.STable3DNode_swigregister(STable3DNode)

class STable3D(object):
    r"""Proxy of C++ mfem::STable3D class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, nr):
        r"""__init__(STable3D self, int nr) -> STable3D"""
        _stable3d.STable3D_swiginit(self, _stable3d.new_STable3D(nr))

    def Push(self, r, c, f):
        r"""Push(STable3D self, int r, int c, int f) -> int"""
        return _stable3d.STable3D_Push(self, r, c, f)
    Push = _swig_new_instance_method(_stable3d.STable3D_Push)

    def Index(self, r, c, f):
        r"""Index(STable3D self, int r, int c, int f) -> int"""
        return _stable3d.STable3D_Index(self, r, c, f)
    Index = _swig_new_instance_method(_stable3d.STable3D_Index)

    def Push4(self, r, c, f, t):
        r"""Push4(STable3D self, int r, int c, int f, int t) -> int"""
        return _stable3d.STable3D_Push4(self, r, c, f, t)
    Push4 = _swig_new_instance_method(_stable3d.STable3D_Push4)

    def __call__(self, *args):
        r"""
        __call__(STable3D self, int r, int c, int f) -> int
        __call__(STable3D self, int r, int c, int f, int t) -> int
        """
        return _stable3d.STable3D___call__(self, *args)
    __call__ = _swig_new_instance_method(_stable3d.STable3D___call__)

    def NumberOfElements(self):
        r"""NumberOfElements(STable3D self) -> int"""
        return _stable3d.STable3D_NumberOfElements(self)
    NumberOfElements = _swig_new_instance_method(_stable3d.STable3D_NumberOfElements)
    __swig_destroy__ = _stable3d.delete_STable3D

    def Print(self, *args):
        r"""
        Print(STable3D self, std::ostream & out=out)
        Print(STable3D self, char const * file, int precision=8)
        """
        return _stable3d.STable3D_Print(self, *args)
    Print = _swig_new_instance_method(_stable3d.STable3D_Print)

# Register STable3D in _stable3d:
_stable3d.STable3D_swigregister(STable3D)






'''
   MFEM example 9
      This is a version of Example 1 with a simple adaptive mesh
      refinement loop. 
      See c++ version in the MFEM library for more detail 
'''
from mfem import path
import mfem.ser as mfem
from mfem.ser import intArray
from os.path import expanduser, join
import numpy as np
from numpy import sqrt, pi, cos, sin, hypot, arctan2
from scipy.special import erfc


problem = 0
ref_levels = 1
order = 1
ode_solver_type = 4
t_final = 10
dt = 0.01
vis_steps = 5

# 2. Read the mesh from the given mesh file. We can handle geometrically
#    periodic meshes in this code.

meshfile = expanduser(join(path, 'data', 'periodic-hexagon.mesh'))
mesh = mfem.Mesh(meshfile, 1,1)
dim = mesh.Dimension()

# 3. Define the ODE solver used for time integration. Several explicit
#    Runge-Kutta methods are available.
ode_solver = None
if ode_solver_type == 1:   ode_solver = mfem.ForwardEulerSolver()
elif ode_solver_type == 2: ode_solver = mfem.RK2Solver(1.0)
elif ode_solver_type == 3: ode_solver = mfem.RK3SSolver()
elif ode_solver_type == 4: ode_solver = mfem.RK4Solver()
elif ode_solver_type == 6: ode_solver = mfem.RK6Solver()
else:
    print( "Unknown ODE solver type: " + str(ode_solver_type))
    exit
    
# 4. Refine the mesh to increase the resolution. In this example we do
#    'ref_levels' of uniform refinement, where 'ref_levels' is a
#    command-line parameter. If the mesh is of NURBS type, we convert it to
#    a (piecewise-polynomial) high-order mesh.
for lev in range(ref_levels):
    mesh.UniformRefinement();
    if mesh.NURBSext:
        mesh.SetCurvature(max(order, 1))
    bb_min, bb_max = mesh.GetBoundingBox(max(order, 1));

# 5. Define the discontinuous DG finite element space of the given
#    polynomial order on the refined mesh.
fec = mfem.DG_FECollection(order, dim)
fes = mfem.FiniteElementSpace(mesh, fec)

print("Number of unknowns: " + str(fes.GetVSize()))

#
#  Define coefficient using VecotrPyCoefficient and PyCoefficient
#  A user needs to define EvalValue method        
#        
class velocity_coeff(mfem.VectorPyCoefficient):
   def EvalValue(self, x):        
       dim = len(x)
        
       center = (bb_min + bb_max)/2.0
       # map to the reference [-1,1] domain                
       X = 2 * (x - center) / (bb_max - bb_min)
       if problem == 0:
           if dim == 1: v = [1.0,]
           elif dim == 2: v = [sqrt(2./3.), sqrt(1./3)]
           elif dim == 3: v = [sqrt(3./6.), sqrt(2./6), sqrt(1./6.)]
       elif (problem == 1 or problem == 2):
           # Clockwise rotation in 2D around the origin                
           w = pi/2
           if dim == 1: v = [1.0,]
           elif dim == 2: v = [w*X[1],  - w*X[0]]
           elif dim == 3: v = [w*X[1],  - w*X[0],  0]
       elif (problem == 3):
           # Clockwise twisting rotation in 2D around the origin
           w = pi/2
           d = max((X[0]+1.)*(1.-X[0]),0.) * max((X[1]+1.)*(1.-X[1]),0.)
           d = d ** 2
           if dim == 1: v = [1.0,]
           elif dim == 2: v = [d*w*X[1],  - d*w*X[0]]
           elif dim == 3: v = [d*w*X[1],  - d*w*X[0],  0]
       return v
        
class u0_coeff(mfem.PyCoefficient):
   def EvalValue(self, x):        
       dim = len(x)

       center = (bb_min + bb_max)/2.0
       # map to the reference [-1,1] domain        
       X = 2 * (x - center) / (bb_max - bb_min)
       if (problem == 0 or problem == 1):
           if dim == 1:
               return exp(-40. * (X[0]-0.5)**2)
           elif (dim == 2 or dim == 3):
               rx = 0.45; ry = 0.25; cx = 0.; cy = -0.2; w = 10.
               if dim == 3:
                   s = (1. + 0.25*cos(2 * pi * x[2]))
                   rx = rx * s
                   ry = ry * s
               return ( erfc( w * (X[0]-cx-rx)) * erfc(-w*(X[0]-cx+rx)) *
                        erfc( w * (X[1]-cy-ry)) * erfc(-w*(X[1]-cy+ry)) )/16
        
       elif problem == 2:
           rho = hypot(x[0], x[1])
           phi = arctan2(x[1], x[0])
           return (sin(pi * rho) **2) * sin(3*phi)
       elif problem == 3:
           return sin(pi * X[0]) * sin(pi * X[1])
       
       return 0.0

# Inflow boundary condition (zero for the problems considered in this example)
class inflow_coeff(mfem.PyCoefficient):
   def EvalValue(self, x):
       return 0

# 6. Set up and assemble the bilinear and linear forms corresponding to the
#    DG discretization. The DGTraceIntegrator involves integrals over mesh
#    interior faces.

velocity = velocity_coeff(dim)
inflow = inflow_coeff()
u0 = u0_coeff()

m = mfem.BilinearForm(fes)
m.AddDomainIntegrator(mfem.MassIntegrator())
k = mfem.BilinearForm(fes)
k.AddDomainIntegrator(mfem.ConvectionIntegrator(velocity, -1.0))
k.AddInteriorFaceIntegrator(
      mfem.TransposeIntegrator(mfem.DGTraceIntegrator(velocity, 1.0, -0.5)))
k.AddBdrFaceIntegrator(
      mfem.TransposeIntegrator(mfem.DGTraceIntegrator(velocity, 1.0, -0.5)))

b = mfem.LinearForm(fes)
b.AddBdrFaceIntegrator(
      mfem.BoundaryFlowIntegrator(inflow, velocity, -1.0, -0.5))

m.Assemble()
m.Finalize()
skip_zeros = 0
k.Assemble(skip_zeros)
k.Finalize(skip_zeros)
b.Assemble()

# 7. Define the initial conditions, save the corresponding grid function to
#    a file 
u = mfem.GridFunction(fes)
u.ProjectCoefficient(u0)

mesh.Print('ex9.mesh', 8)
u.Save('ex9-init.gf', 8)

class FE_Evolution(mfem.PyTimeDependentOperator):
    def __init__(self, M, K, b):
        mfem.PyTimeDependentOperator.__init__(self, M.Size())        

        
        self.K = K        
        self.M = M
        self.b = b
        self.z = mfem.Vector(M.Size())
        self.zp = np.zeros(M.Size())
        self.M_prec = mfem.DSmoother()        
        self.M_solver = mfem.CGSolver()
        self.M_solver.SetPreconditioner(self.M_prec)        
        self.M_solver.SetOperator(M)
        self.M_solver.iterative_mode = False
        self.M_solver.SetRelTol(1e-9)
        self.M_solver.SetAbsTol(0.0)
        self.M_solver.SetMaxIter(100)
        self.M_solver.SetPrintLevel(0)
        

#    def EvalMult(self, x):
#        if you want to impolement Mult in using python objects, 
#        such as numpy.. this needs to be implemented and don't
#        overwrite Mult

    def Mult(self, x, y):
        self.K.Mult(x, self.z);
        self.z += b;
        self.M_solver.Mult(self.z, y)

adv = FE_Evolution(m.SpMat(), k.SpMat(), b)

ode_solver.Init(adv)
t = 0.0; ti = 0;
while True:
   if t > t_final - dt/2: break
   t, dt = ode_solver.Step(u, t, dt);
   ti = ti + 1
   if ti % vis_steps == 0:
       print("time step: " + str(ti) + ", time: " + str(t))

u.Save('ex9-final.gf', 8)



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _estimators
else:
    import _estimators

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _estimators.SWIG_PyInstanceMethod_New
_swig_new_static_method = _estimators.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

MFEM_VERSION = _estimators.MFEM_VERSION
MFEM_VERSION_STRING = _estimators.MFEM_VERSION_STRING
MFEM_VERSION_TYPE = _estimators.MFEM_VERSION_TYPE
MFEM_VERSION_TYPE_RELEASE = _estimators.MFEM_VERSION_TYPE_RELEASE
MFEM_VERSION_TYPE_DEVELOPMENT = _estimators.MFEM_VERSION_TYPE_DEVELOPMENT
MFEM_VERSION_MAJOR = _estimators.MFEM_VERSION_MAJOR
MFEM_VERSION_MINOR = _estimators.MFEM_VERSION_MINOR
MFEM_VERSION_PATCH = _estimators.MFEM_VERSION_PATCH
MFEM_HYPRE_VERSION = _estimators.MFEM_HYPRE_VERSION
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.vector
import mfem._par.fespace
import mfem._par.coefficient
import mfem._par.globals
import mfem._par.matrix
import mfem._par.operators
import mfem._par.intrules
import mfem._par.sparsemat
import mfem._par.densemat
import mfem._par.eltrans
import mfem._par.fe
import mfem._par.geom
import mfem._par.mesh
import mfem._par.sort_pairs
import mfem._par.ncmesh
import mfem._par.vtk
import mfem._par.element
import mfem._par.table
import mfem._par.hash
import mfem._par.vertex
import mfem._par.gridfunc
import mfem._par.bilininteg
import mfem._par.fe_coll
import mfem._par.lininteg
import mfem._par.linearform
import mfem._par.handle
import mfem._par.hypre
import mfem._par.restriction
import mfem._par.bilinearform
class AbstractErrorEstimator(object):
    r"""Proxy of C++ mfem::AbstractErrorEstimator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    __swig_destroy__ = _estimators.delete_AbstractErrorEstimator

    def __init__(self):
        r"""__init__(AbstractErrorEstimator self) -> AbstractErrorEstimator"""
        _estimators.AbstractErrorEstimator_swiginit(self, _estimators.new_AbstractErrorEstimator())

# Register AbstractErrorEstimator in _estimators:
_estimators.AbstractErrorEstimator_swigregister(AbstractErrorEstimator)

class ErrorEstimator(AbstractErrorEstimator):
    r"""Proxy of C++ mfem::ErrorEstimator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def GetLocalErrors(self):
        r"""GetLocalErrors(ErrorEstimator self) -> Vector"""
        return _estimators.ErrorEstimator_GetLocalErrors(self)
    GetLocalErrors = _swig_new_instance_method(_estimators.ErrorEstimator_GetLocalErrors)

    def Reset(self):
        r"""Reset(ErrorEstimator self)"""
        return _estimators.ErrorEstimator_Reset(self)
    Reset = _swig_new_instance_method(_estimators.ErrorEstimator_Reset)
    __swig_destroy__ = _estimators.delete_ErrorEstimator

# Register ErrorEstimator in _estimators:
_estimators.ErrorEstimator_swigregister(ErrorEstimator)

class AnisotropicErrorEstimator(ErrorEstimator):
    r"""Proxy of C++ mfem::AnisotropicErrorEstimator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr

    def GetAnisotropicFlags(self):
        r"""GetAnisotropicFlags(AnisotropicErrorEstimator self) -> intArray"""
        return _estimators.AnisotropicErrorEstimator_GetAnisotropicFlags(self)
    GetAnisotropicFlags = _swig_new_instance_method(_estimators.AnisotropicErrorEstimator_GetAnisotropicFlags)
    __swig_destroy__ = _estimators.delete_AnisotropicErrorEstimator

# Register AnisotropicErrorEstimator in _estimators:
_estimators.AnisotropicErrorEstimator_swigregister(AnisotropicErrorEstimator)

class ZienkiewiczZhuEstimator(AnisotropicErrorEstimator):
    r"""Proxy of C++ mfem::ZienkiewiczZhuEstimator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def SetWithCoeff(self, w_coeff=True):
        r"""SetWithCoeff(ZienkiewiczZhuEstimator self, bool w_coeff=True)"""
        return _estimators.ZienkiewiczZhuEstimator_SetWithCoeff(self, w_coeff)
    SetWithCoeff = _swig_new_instance_method(_estimators.ZienkiewiczZhuEstimator_SetWithCoeff)

    def SetAnisotropic(self, aniso=True):
        r"""SetAnisotropic(ZienkiewiczZhuEstimator self, bool aniso=True)"""
        return _estimators.ZienkiewiczZhuEstimator_SetAnisotropic(self, aniso)
    SetAnisotropic = _swig_new_instance_method(_estimators.ZienkiewiczZhuEstimator_SetAnisotropic)

    def SetFluxAveraging(self, fa):
        r"""SetFluxAveraging(ZienkiewiczZhuEstimator self, int fa)"""
        return _estimators.ZienkiewiczZhuEstimator_SetFluxAveraging(self, fa)
    SetFluxAveraging = _swig_new_instance_method(_estimators.ZienkiewiczZhuEstimator_SetFluxAveraging)

    def GetTotalError(self):
        r"""GetTotalError(ZienkiewiczZhuEstimator self) -> double"""
        return _estimators.ZienkiewiczZhuEstimator_GetTotalError(self)
    GetTotalError = _swig_new_instance_method(_estimators.ZienkiewiczZhuEstimator_GetTotalError)

    def GetLocalErrors(self):
        r"""GetLocalErrors(ZienkiewiczZhuEstimator self) -> Vector"""
        return _estimators.ZienkiewiczZhuEstimator_GetLocalErrors(self)
    GetLocalErrors = _swig_new_instance_method(_estimators.ZienkiewiczZhuEstimator_GetLocalErrors)

    def GetAnisotropicFlags(self):
        r"""GetAnisotropicFlags(ZienkiewiczZhuEstimator self) -> intArray"""
        return _estimators.ZienkiewiczZhuEstimator_GetAnisotropicFlags(self)
    GetAnisotropicFlags = _swig_new_instance_method(_estimators.ZienkiewiczZhuEstimator_GetAnisotropicFlags)

    def Reset(self):
        r"""Reset(ZienkiewiczZhuEstimator self)"""
        return _estimators.ZienkiewiczZhuEstimator_Reset(self)
    Reset = _swig_new_instance_method(_estimators.ZienkiewiczZhuEstimator_Reset)
    __swig_destroy__ = _estimators.delete_ZienkiewiczZhuEstimator

    def __init__(self, integ, sol, flux_fes, own_flux_fes=False):
        r"""__init__(ZienkiewiczZhuEstimator self, BilinearFormIntegrator integ, GridFunction sol, FiniteElementSpace flux_fes, bool own_flux_fes=False) -> ZienkiewiczZhuEstimator"""

        if own_flux_fes: flux_fes.thisown=0


        _estimators.ZienkiewiczZhuEstimator_swiginit(self, _estimators.new_ZienkiewiczZhuEstimator(integ, sol, flux_fes, own_flux_fes))

# Register ZienkiewiczZhuEstimator in _estimators:
_estimators.ZienkiewiczZhuEstimator_swigregister(ZienkiewiczZhuEstimator)

class L2ZienkiewiczZhuEstimator(ErrorEstimator):
    r"""Proxy of C++ mfem::L2ZienkiewiczZhuEstimator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def SetLocalErrorNormP(self, p):
        r"""SetLocalErrorNormP(L2ZienkiewiczZhuEstimator self, int p)"""
        return _estimators.L2ZienkiewiczZhuEstimator_SetLocalErrorNormP(self, p)
    SetLocalErrorNormP = _swig_new_instance_method(_estimators.L2ZienkiewiczZhuEstimator_SetLocalErrorNormP)

    def GetTotalError(self):
        r"""GetTotalError(L2ZienkiewiczZhuEstimator self) -> double"""
        return _estimators.L2ZienkiewiczZhuEstimator_GetTotalError(self)
    GetTotalError = _swig_new_instance_method(_estimators.L2ZienkiewiczZhuEstimator_GetTotalError)

    def GetLocalErrors(self):
        r"""GetLocalErrors(L2ZienkiewiczZhuEstimator self) -> Vector"""
        return _estimators.L2ZienkiewiczZhuEstimator_GetLocalErrors(self)
    GetLocalErrors = _swig_new_instance_method(_estimators.L2ZienkiewiczZhuEstimator_GetLocalErrors)

    def Reset(self):
        r"""Reset(L2ZienkiewiczZhuEstimator self)"""
        return _estimators.L2ZienkiewiczZhuEstimator_Reset(self)
    Reset = _swig_new_instance_method(_estimators.L2ZienkiewiczZhuEstimator_Reset)
    __swig_destroy__ = _estimators.delete_L2ZienkiewiczZhuEstimator

    def __init__(self, integ, sol, flux_fes, smooth_flux_fes, own_flux_fes=False):
        r"""__init__(L2ZienkiewiczZhuEstimator self, BilinearFormIntegrator integ, mfem::ParGridFunction & sol, mfem::ParFiniteElementSpace * flux_fes, mfem::ParFiniteElementSpace * smooth_flux_fes, bool own_flux_fes=False) -> L2ZienkiewiczZhuEstimator"""

        if own_flux_fes: flux_fes.thisown=0


        _estimators.L2ZienkiewiczZhuEstimator_swiginit(self, _estimators.new_L2ZienkiewiczZhuEstimator(integ, sol, flux_fes, smooth_flux_fes, own_flux_fes))

# Register L2ZienkiewiczZhuEstimator in _estimators:
_estimators.L2ZienkiewiczZhuEstimator_swigregister(L2ZienkiewiczZhuEstimator)

class LpErrorEstimator(ErrorEstimator):
    r"""Proxy of C++ mfem::LpErrorEstimator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(LpErrorEstimator self, int p, GridFunction sol) -> LpErrorEstimator
        __init__(LpErrorEstimator self, int p, Coefficient coef, GridFunction sol) -> LpErrorEstimator
        __init__(LpErrorEstimator self, int p, VectorCoefficient coef, GridFunction sol) -> LpErrorEstimator
        """
        _estimators.LpErrorEstimator_swiginit(self, _estimators.new_LpErrorEstimator(*args))

    def SetLocalErrorNormP(self, p):
        r"""SetLocalErrorNormP(LpErrorEstimator self, int p)"""
        return _estimators.LpErrorEstimator_SetLocalErrorNormP(self, p)
    SetLocalErrorNormP = _swig_new_instance_method(_estimators.LpErrorEstimator_SetLocalErrorNormP)

    def SetCoef(self, *args):
        r"""
        SetCoef(LpErrorEstimator self, Coefficient A)
        SetCoef(LpErrorEstimator self, VectorCoefficient A)
        """
        return _estimators.LpErrorEstimator_SetCoef(self, *args)
    SetCoef = _swig_new_instance_method(_estimators.LpErrorEstimator_SetCoef)

    def Reset(self):
        r"""Reset(LpErrorEstimator self)"""
        return _estimators.LpErrorEstimator_Reset(self)
    Reset = _swig_new_instance_method(_estimators.LpErrorEstimator_Reset)

    def GetLocalErrors(self):
        r"""GetLocalErrors(LpErrorEstimator self) -> Vector"""
        return _estimators.LpErrorEstimator_GetLocalErrors(self)
    GetLocalErrors = _swig_new_instance_method(_estimators.LpErrorEstimator_GetLocalErrors)
    __swig_destroy__ = _estimators.delete_LpErrorEstimator

# Register LpErrorEstimator in _estimators:
_estimators.LpErrorEstimator_swigregister(LpErrorEstimator)






from __future__ import print_function
import os
import sys
import numpy as np

if len(sys.argv) > 1 and sys.argv[1] == '-p':   
    import mfem.par as mfem
    use_parallel = True
    from mfem.common.mpi_debug import nicePrint as print
    from mpi4py import MPI
    myid  = MPI.COMM_WORLD.rank
    
else:
    import mfem.ser as mfem
    use_parallel = False
    myid = 0
    
def run_test(): 
    m = mfem.DenseMatrix(3,3)
    m.Assign(np.arange(9.).reshape(3,3))
    m.Print()
    print(np.arange(9.).reshape(3,3))
    
    x = np.zeros(5)+1
    y = np.zeros(5)+2
    z = np.zeros(5)+3
    mm = mfem.DenseMatrix(3, 5)
    mm.Assign(np.vstack([x, y, z]))
    mm.Print()
    mfem.Vector(mm.GetData(), 15).Print()

    mm.Print("densmat.dat")
    
if __name__=='__main__':
    run_test()
 



#!/usr/bin/env python

"""
setup.py file for SWIG example
"""
from distutils.core import setup, Extension
import os
from os.path import expanduser

from distutils.core import *
from distutils      import sysconfig

import numpy
try:
    numpy_include = numpy.get_include()
except AttributeError:
     numpy_include = numpy.get_numpy_include()

home = expanduser("~")
mfem_dir = os.path.join(home, 'mfem-3.1')+'/'
mfem_lib ='mfem'

mfem_module = Extension('_mfem',
                        sources=['mfem_wrap.cxx'],
                        include_dirs=[mfem_dir],
                        library_dirs=[mfem_dir],
                        libraries=[mfem_lib])
setup (name = 'mfem',
       version = '0.1',
       author      = "S.Shiraiwa",
       description = """MFEM wrapper""",
       ext_modules = [mfem_module,],
       py_modules = ["mfem",]
       )
